@@ -19,6 +19,7 @@ MATRIX_PLATFORMS_LOOKUP = {"enterprise-attack": ['PRE', 'Windows', 'macOS', 'Lin
                                           "Control Server", "Input/Output Server", "Windows", "Human-Machine Interface",
                                           "Engineering Workstation", "Data Historian"]}
 
+TITLE_EXCLUSIONS = ['and']
 
 def remove_revoked_deprecated(stix_objects):
     """Remove any revoked or deprecated objects from queries made to the data source"""
@@ -127,7 +128,8 @@ def techniquesToDf(src, domain):
 
         # sub-technique properties
         tactic_shortnames = list(map(lambda kcp: kcp["phase_name"], technique["kill_chain_phases"]))
-        tactics = list(map(lambda t: t.replace("-", " ").title(), tactic_shortnames))
+        tactics = list(map(lambda t: ' '.join([x.title() if x not in TITLE_EXCLUSIONS else x for x in t.split('-')]),
+                           tactic_shortnames))
         row["tactics"] = ", ".join(sorted(tactics))
 
         if "x_mitre_detection" in technique:
@@ -149,17 +151,23 @@ def techniquesToDf(src, domain):
 
             if "x_mitre_system_requirements" in technique:
                 row["system requirements"] = ", ".join(sorted(technique["x_mitre_system_requirements"]))
-            if "privilege-escalation" in tactic_shortnames:
-                if "x_mitre_permissions_required" in technique:
-                    row["permissions required"] = ", ".join(sorted(technique["x_mitre_permissions_required"]))
-                if "x_mitre_effective_permissions" in technique:
-                    row["effective permissions"] = ", ".join(sorted(technique["x_mitre_effective_permissions"]))
+            if "x_mitre_permissions_required" in technique:
+                row["permissions required"] = ", ".join(sorted(technique["x_mitre_permissions_required"],
+                                                               key=str.lower))
+            if "x_mitre_effective_permissions" in technique:
+                row["effective permissions"] = ", ".join(sorted(technique["x_mitre_effective_permissions"],
+                                                                key=str.lower))
+
             if "defense-evasion" in tactic_shortnames and "x_mitre_defense_bypassed" in technique:
                 row["defenses bypassed"] = ", ".join(sorted(technique["x_mitre_defense_bypassed"]))
             if "execution" in tactic_shortnames and "x_mitre_remote_support" in technique:
                 row["supports remote"] = technique["x_mitre_remote_support"]
             if "impact" in tactic_shortnames and "x_mitre_impact_type" in technique:
                 row["impact type"] = ", ".join(sorted(technique["x_mitre_impact_type"]))
+            capec_refs = list(filter(lambda ref: ref["source_name"] == "capec",
+                                     technique["external_references"]))
+            if capec_refs:
+                row["CAPEC ID"] = ", ".join([x["external_id"] for x in capec_refs])
 
         # domain specific fields -- mobile
         elif domain == "mobile-attack":
@@ -177,7 +185,10 @@ def techniquesToDf(src, domain):
         "techniques": pd.DataFrame(technique_rows).sort_values("name"),
     }
     # add relationships
-    dataframes.update(relationshipsToDf(src, relatedType="technique"))
+    codex = relationshipsToDf(src, relatedType="technique")
+    dataframes.update(codex)
+    # add relationship references
+    dataframes["techniques"][f"relationship citations"] = _get_relationship_citations(dataframes['techniques'], codex)
     # add/merge citations
     if not citations.empty:
         if "citations" in dataframes:  # append to existing citations from references
@@ -225,49 +236,52 @@ def sourcesToDf(src, domain):
         src.query(f) for f in [Filter("type", "=", "x-mitre-data-component"),
                                Filter("type", "=", "x-mitre-data-source")]
     ))
-    refined = remove_revoked_deprecated(data)
-    data_object_rows = []
-    source_lookup = dict()
-    for x in refined:
-        if x['type'] == 'x-mitre-data-source':
-            source_lookup[x['id']] = x['name']
-    for data_object in tqdm(refined, desc="parsing data sources"):
-        # add common STIx fields
-        row = parseBaseStix(data_object)
-        # add software-specific fields
-        if "x_mitre_platforms" in data_object:
-            row["platforms"] = ", ".join(sorted(data_object["x_mitre_platforms"]))
-        if "x_mitre_collection_layers" in data_object:
-            row["collection layers"] = ', '.join(sorted(data_object["x_mitre_collection_layers"]))
-        if "x_mitre_aliases" in data_object:
-            row["aliases"] = ", ".join(sorted(data_object["x_mitre_aliases"][1:]))
-        if data_object["type"] == "x-mitre-data-component":
-            row["name"] = f"{data_object['name']}: {source_lookup[data_object['x_mitre_data_source_ref']]}"
-            row["type"] = "datacomponent"
-        else:
-            row["type"] = "datasource"
-        if "description" in data_object:
-            row['description'] = data_object['description']
-        data_object_rows.append(row)
+    if data:
+        refined = remove_revoked_deprecated(data)
+        data_object_rows = []
+        source_lookup = dict()
+        for x in refined:
+            if x['type'] == 'x-mitre-data-source':
+                source_lookup[x['id']] = x['name']
+        for data_object in tqdm(refined, desc="parsing data sources"):
+            # add common STIx fields
+            row = parseBaseStix(data_object)
+            # add software-specific fields
+            if "x_mitre_platforms" in data_object:
+                row["platforms"] = ", ".join(sorted(data_object["x_mitre_platforms"]))
+            if "x_mitre_collection_layers" in data_object:
+                row["collection layers"] = ', '.join(sorted(data_object["x_mitre_collection_layers"]))
+            if "x_mitre_aliases" in data_object:
+                row["aliases"] = ", ".join(sorted(data_object["x_mitre_aliases"][1:]))
+            if data_object["type"] == "x-mitre-data-component":
+                row["name"] = f"{data_object['name']}: {source_lookup[data_object['x_mitre_data_source_ref']]}"
+                row["type"] = "datacomponent"
+            else:
+                row["type"] = "datasource"
+            if "description" in data_object:
+                row['description'] = data_object['description']
+            data_object_rows.append(row)
 
-    citations = get_citations(refined)
-    tempa = pd.DataFrame(data_object_rows).sort_values("name")
-    dataframes = {
-        "datasources": tempa.reindex(columns=["name", "ID", "description", "collection layers", "platforms", "created",
-                                               "modified", "type", "version", "url", "contributors"]),
-    }
-    # add relationships
-    dataframes.update(relationshipsToDf(src, relatedType="datasource"))
-    # add/merge citations
-    if not citations.empty:
-        if "citations" in dataframes:  # append to existing citations from references
-            dataframes["citations"] = dataframes["citations"].append(citations)
-        else:  # add citations
-            dataframes["citations"] = citations
+        citations = get_citations(refined)
+        tempa = pd.DataFrame(data_object_rows).sort_values("name")
+        dataframes = {
+            "datasources": tempa.reindex(columns=["name", "ID", "description", "collection layers", "platforms", "created",
+                                                   "modified", "type", "version", "url", "contributors"]),
+        }
+        # add relationships
+        dataframes.update(relationshipsToDf(src, relatedType="datasource"))
+        # add/merge citations
+        if not citations.empty:
+            if "citations" in dataframes:  # append to existing citations from references
+                dataframes["citations"] = dataframes["citations"].append(citations)
+            else:  # add citations
+                dataframes["citations"] = citations
 
-        dataframes["citations"].sort_values("reference")
+            dataframes["citations"].sort_values("reference")
 
-    return dataframes
+        return dataframes
+    else:
+        print(f'[WARNING] (sourceToDf) - No data components or data sources found for domain {domain}. Skipping...')
 
 
 def softwareToDf(src, domain):
@@ -300,7 +314,10 @@ def softwareToDf(src, domain):
         "software": pd.DataFrame(software_rows).sort_values("name"),
     }
     # add relationships
-    dataframes.update(relationshipsToDf(src, relatedType="software"))
+    codex = relationshipsToDf(src, relatedType="software")
+    dataframes.update(codex)
+    # add relationship references
+    dataframes["software"][f"relationship citations"] = _get_relationship_citations(dataframes["software"], codex)
     # add/merge citations
     if not citations.empty:
         if "citations" in dataframes:  # append to existing citations from references
@@ -348,7 +365,10 @@ def groupsToDf(src, domain):
         "groups": pd.DataFrame(group_rows).sort_values("name"),
     }
     # add relationships
-    dataframes.update(relationshipsToDf(src, relatedType="group"))
+    codex = relationshipsToDf(src, relatedType="group")
+    dataframes.update(codex)
+    # add relationship references
+    dataframes["groups"][f"relationship citations"] = _get_relationship_citations(dataframes["groups"], codex)
     # add/merge citations
     if not citations.empty:
         if "citations" in dataframes:  # append to existing citations from references
@@ -380,7 +400,10 @@ def mitigationsToDf(src, domain):
         "mitigations": pd.DataFrame(mitigation_rows).sort_values("name"),
     }
     # add relationships
-    dataframes.update(relationshipsToDf(src, relatedType="mitigation"))
+    codex = relationshipsToDf(src, relatedType="mitigation")
+    dataframes.update(codex)
+    # add relationship references
+    dataframes["mitigations"]["relationship citations"] = _get_relationship_citations(dataframes["mitigations"], codex)
     # add/merge citations
     if not citations.empty:
         if "citations" in dataframes:  # append to existing citations from references
@@ -711,7 +734,7 @@ def relationshipsToDf(src, relatedType=None):
     relationships = pd.DataFrame(relationship_rows).sort_values(
         ["mapping type", "source type", "target type", "source name", "target name"])
 
-    if not relatedType:  # return all relationships and citrations
+    if not relatedType:  # return all relationships and citations
         dataframes = {
             "relationships": relationships,
         }
@@ -741,7 +764,7 @@ def relationshipsToDf(src, relatedType=None):
                 'associated mitigations' if relatedType == 'technique' else 'techniques addressed'] = relatedMitigations
 
         if not citations.empty:
-            # fitler citations by ones actually used
+            # filter citations by ones actually used
             # build master list of used citations
             usedCitations = set()
             for dfname in dataframes:
@@ -755,4 +778,32 @@ def relationshipsToDf(src, relatedType=None):
 
             dataframes["citations"] = citations.sort_values("reference")
 
-        return dataframes
\ No newline at end of file
+        return dataframes
+
+
+def _get_relationship_citations(object_dataframe, relationship_df):
+    """
+    Extract citations for each _object_ in the relationship dataframe. This allows us to include
+    citations from relationships for each ATT&CK object type.
+    :param object_dataframe: Dataframe for relevant ATT&CK object
+    :param relationship_df: Dataframe of relationships
+    :return: Array of strings, with each string being placed relative to the object listing, and containing all
+        relevant citations
+    """
+    object_listing = [x for x in object_dataframe["ID"]]
+    new_citations = []
+    for z in [x for x in relationship_df if x != 'citations']:
+        subset = []
+        for y in object_listing:
+            mask = relationship_df[z].values == y
+            filtered = relationship_df[z].loc[mask]
+            temp = set()
+            for description in filter(lambda x: x == x, filtered["mapping description"].tolist()):
+                [temp.add(x) for x in re.findall(r"\(Citation: (.*?)\)", description)]
+            subset.append(",".join([f"(Citation: {z})" for z in temp]))
+        if not new_citations:
+            new_citations = subset
+        else:
+            for i in range(0, len(new_citations)):
+                new_citations[i] = ','.join([new_citations[i], subset[i]])
+    return new_citations
